<?php

defined( 'ABSPATH' ) || exit;

if ( ! class_exists( 'Pwf_Woo_Main_Query' ) ) {

	class Pwf_Woo_Main_Query {

		/**
		* The unique instance of the plugin.
		*
		* @var Pwf_Woo_Main_Query
		*/
		private static $instance;

		/**
		* The unique instance of the Pwf_Parse_Query_Vars.
		*
		* @var Pwf_Parse_Query_Vars
		*/
		private static $query_vars;

		private $filter_id;
		private $query_type;
		private $filter_items;
		private $filter_settings;
		private $global_args;

		/**
		* hook price
		*/
		private static $hook_price;

		/**
		* hook orderby
		*/
		private static $hook_orderby;

		/**
		 * check if analtic add before
		 * On main query analytic can add more times
		 * This variable prevent form add twics
		 */
		private static $is_analytic_add = false;

		private static $page_number;

		/**
		 * @since 1.0.0, 1.0.6, 1.5.4
		 */
		private function __construct() {
			self::$query_vars   = null;
			self::$hook_price   = false;
			self::$hook_orderby = false;

			add_action( 'pre_get_posts', array( $this, 'woocommerce_product_query' ), 20, 1 );
		}

		/**
		 * Gets an instance of our plugin.
		 *
		 * @return Pwf_Woo_Main_Query
		 */
		public static function get_instance() {
			if ( null === self::$instance ) {
				self::$instance = new self();
			}

			return self::$instance;
		}

		public function woocommerce_product_query( $q ) {
			$filter_id = $this->get_filter_id( $q );

			if ( empty( $filter_id ) ) {
				return;
			}

			$this->set_filter_variables();

			if ( empty( $this->filter_items ) ) {
				return;
			}

			if ( 'main_query' === $this->query_type ) {
				$this->prepare_target_query( $q );
			} else {
				add_action( 'pre_get_posts', array( $this, 'prepare_custom_query' ), 10, 1 );
			}
		}

		protected function get_filter_id( $q ) {
			$filter_id  = '';
			$query_type = '';
			$page_type  = '';
			$page_id    = '';

			if ( is_post_type_archive( 'product' ) || is_tax( get_object_taxonomies( 'product' ) ) ) {
				if ( $q->is_main_query() ) {
					if ( is_tax( get_object_taxonomies( 'product' ) ) ) {
						$query_obj = $q->get_queried_object();
						$page_id   = $query_obj->term_id;
						$page_type = $query_obj->taxonomy;
						$filter_id = self::get_page_filter_id( 'tax', $page_id, $page_type );
					} else {
						$page_type = 'shop';
						$page_id   = absint( get_option( 'woocommerce_shop_page_id' ) );
						$filter_id = self::get_page_filter_id( 'shop' );
					}
					$query_type = 'main_query';
				}
			} elseif ( is_page() ) {
				$page_type = 'page';
				$page      = $q->get_queried_object();
				if ( isset( $page->ID ) ) {
					$page_id    = $page->ID;
					$filter_id  = self::get_page_filter_id( 'page', $page_id );
					$query_type = 'custom_query';
				}
			}

			if ( empty( $filter_id ) ) {
				/**
				 * check divi shop builder used shortcode abd doesn't hook main query
				 */
				if ( class_exists( 'AGS_divi_wc' ) ) {
					if ( is_shop() ) {
						$page_id    = get_option( 'woocommerce_shop_page_id' );
						$filter_id  = self::get_page_filter_id( 'page', $page_id );
						$query_type = 'custom_query';
						Pwf_Clear_Transients::delete_transients( '_transient_timeout_wc_product_loop_' );
					}
				}
			}

			if ( ! empty( $filter_id ) ) {
				$filter_id         = apply_filters( 'pwf_filter_id', $filter_id );
				$this->filter_id   = $filter_id;
				$this->query_type  = $query_type;
				$this->global_args = array(
					'filter_id'         => $filter_id,
					'query_type'        => $query_type,
					'current_page_id'   => $page_id,
					'current_page_type' => $page_type,
					'is_shop_archive'   => ( 'page' === $page_type ) ? 'false' : 'true',
				);
			}

			return $filter_id;
		}

		protected static function get_page_filter_id( $type, $term_id = '', $tax_name = '' ) {
			$filter_id = '';
			if ( 'tax' === $type || 'shop' === $type ) {
				$filter_ids = get_option( 'pwf_woo_main_query_filters', '' );
				if ( ! empty( $filter_ids ) && is_array( $filter_ids ) ) {
					$filter_ids = self::convert_terms_to_wpml( $filter_ids, 'archive' );
					$search_in  = array_keys( $filter_ids );
					if ( 'shop' === $type ) {
						if ( in_array( 'shop_archive', $search_in, true ) ) {
							$filter_id = $filter_ids['shop_archive'];
						}
					} else {
						$search_for = $tax_name . '__' . $term_id;
						if ( in_array( $search_for, $search_in, true ) ) {
							$filter_id = $filter_ids[ $search_for ];
						} elseif ( in_array( $tax_name . '__all', $search_in, true ) ) {
							$filter_id = $filter_ids[ $tax_name . '__all' ];
						} elseif ( in_array( 'shop_archive', $search_in, true ) ) {
							$filter_id = $filter_ids['shop_archive'];
						}
					}
				}
			} elseif ( 'page' === $type ) {
				$filter_ids = get_option( 'pwf_woo_custom_query_filters', '' );
				if ( ! empty( $filter_ids ) && is_array( $filter_ids ) ) {
					$filter_ids = self::convert_terms_to_wpml( $filter_ids, 'page' );
					$search_in  = array_map( 'absint', array_keys( $filter_ids ) );
					if ( in_array( absint( $term_id ), $search_in, true ) ) {
						$filter_id = $filter_ids[ $term_id ];
					}
				}
			}

			return $filter_id;
		}

		/**
		 * Convert terms save in options to current langauage
		 * Useuful when make compare against current term is translated
		 *
		 * @param array $terms
		 * @param string $type archive or page
		 *
		 * @since 1.5.7
		 */
		protected static function convert_terms_to_wpml( array $terms, $type ) {
			if ( ! Pwf_Main::is_wpml_defined() ) {
				return $terms;
			}

			$results = array();
			if ( 'archive' === $type ) {
				foreach ( $terms as $key => $filter_id ) {
					if ( 'shop_archive' === $key ) {
						$results[ $key ] = $filter_id;
					} else {
						$split     = explode( '__', $key );
						$tax_name  = $split[0];
						$tax_value = $split[1];
						if ( 'all' === $tax_value ) {
							$results[ $key ] = $filter_id;
						} else {
							$tax_value = Pwf_Wpml::get_translated_term_id( absint( $tax_value ), $tax_name );

							$results[ $tax_name . '__' . $tax_value ] = $filter_id;
						}
					}
				}
			} else {
				foreach ( $terms as $page_id => $filter_id ) {
					$page_id = Pwf_Wpml::get_translated_term_id( absint( $page_id ), 'page' );

					$results[ $page_id ] = $filter_id;
				}
			}

			return $results;
		}

		/**
		 * @since 1.5.4
		 * @param $q wp_query
		 */
		public function prepare_target_query( $q ) {

			$this->global_args['shop_integrated'] = 'yes';

			$GLOBALS['pwf_main_query'] = wp_parse_args( $this->global_args, $GLOBALS['pwf_main_query'] );

			$active_items     = $this->get_active_filter_items();
			self::$query_vars = new Pwf_Parse_Query_Vars( $this->filter_id, $active_items );
			$orderby          = self::$query_vars->get_products_orderby();
			$tax_query        = self::$query_vars->get_tax_query_filter_items();
			$meta_query       = self::$query_vars->get_meta_query();
			$authors_id       = self::$query_vars->get_authors_id();
			$date_query       = self::$query_vars->get_date_query();

			$GLOBALS['pwf_main_query']['query_vars'] = self::$query_vars;

			if ( ! empty( $orderby ) && ! isset( $_GET['orderby'] ) ) {
				self::$hook_orderby = true;

				$orderby  = is_array( $orderby ) ? implode( '', $orderby ) : $orderby;
				$ordering = WC()->query->get_catalog_ordering_args( $orderby );

				$q->set( 'orderby', $ordering['orderby'] );
				$q->set( 'order', $ordering['order'] );
				if ( isset( $ordering['meta_key'] ) ) {
					$q->set( 'meta_key', $ordering['meta_key'] );
				}
			}

			if ( self::$query_vars->has_price_item() ) {
				if ( 'custom_query' === $this->query_type || ( ! isset( $_GET['min_price'] ) && ! isset( $_GET['max_price'] ) ) ) {
					self::$hook_price = true;
					add_filter( 'posts_clauses', array( $this, 'price_filter_post_clauses' ), 10, 2 );
				}
			}

			if ( ! empty( self::$query_vars->get_search_query() ) ) {
				// Get[s] check if woocommerce add these code before in wc_query
				if ( 'custom_query' === $this->query_type || ! isset( $_GET['s'] ) ) {
					$q->set( 's', self::$query_vars->get_search_query() );
					$q->set( 'is_search', true );
				}
				if ( 'only_title' === self::$query_vars->get_search_inside() ) {
					add_filter( 'posts_search', array( $this, 'search_by_title_only' ), 200, 1 );
				}
			}

			if ( self::$query_vars->is_stock_status_active() ) {
				add_filter( 'posts_where', array( $this, 'set_stock_status_product_variations' ), 10, 2 );
			}

			if ( ! empty( self::$query_vars->has_on_sale() ) ) {
				add_filter( 'posts_where', array( $this, 'append_on_sale_products' ), 10, 2 );
			}

			if ( ! empty( $tax_query ) ) {
				if ( ! empty( $q->get( 'tax_query' ) ) ) {
					$tax_query = array_merge( $q->get( 'tax_query' ), $tax_query );
				}
				$q->set( 'tax_query', $tax_query );
			}

			if ( ! empty( $meta_query ) ) {
				$meta_query = array_merge( $q->get( 'meta_query' ), $meta_query );
				$q->set( 'meta_query', $meta_query );
			}

			if ( ! empty( $authors_id ) ) {
				$authors_id = array_merge( $q->get( 'author__in' ), $authors_id );
				$q->set( 'author__in', $authors_id );
			}

			if ( ! empty( $date_query ) ) {
				$q->set( 'date_query', $date_query );
			}

			if ( null !== self::$page_number ) {
				$q->set( 'paged', self::$page_number );
			}

			add_filter( 'the_posts', array( $this, 'remove_product_query_filters' ) );
		}

		public function prepare_custom_query( $q ) {
			if ( 'custom_query' === $this->query_type && $this->is_that_exact_query_requires( $q ) ) {
				self::check_page_number();
				$this->prepare_target_query( $q );
			}
		}

		protected static function check_page_number() {
			global $wp;

			$current_url = home_url( add_query_arg( array(), $wp->request ) );

			// get the position where '/page.. ' text start.
			$pos = strpos( $current_url, '/page' );

			// remove string from the specific postion
			$url = ( $pos ) ? substr( $current_url, $pos ) : $current_url;

			$page_num = preg_replace( '/[^0-9]/', '', $url );

			if ( $page_num ) {
				self::$page_number = absint( $page_num );
			}
		}

		protected function set_filter_variables() {
			if ( Pwf_Main::is_wpml_defined() ) {
				$filter = Pwf_Wpml::get_filter_post_meta( $this->filter_id );
			} else {
				$filter = get_post_meta( absint( $this->filter_id ), '_pwf_woo_post_filter', true );
			}

			if ( ! empty( $filter ) ) {
				$this->filter_items    = $filter['items'];
				$this->filter_settings = $filter['setting'];
			}
		}

		protected function set_global_variables() {
			$this->global_args['query_vars']      = self::$query_vars;
			$this->global_args['shop_integrated'] = 'yes';

			$GLOBALS['pwf_main_query'] = wp_parse_args( $this->global_args, $GLOBALS['pwf_main_query'] );
		}

		/**
		 * Check if that the wp_query requires to hook.
		 * The page can contain many querys to display products
		 * On the forntend like widgets, shortcode, ...
		 * Our plugin check only by limit
		 *
		 * @param object $q wp_query
		 */
		protected function is_that_exact_query_requires( $q ) {
			$post_type = $q->get( 'post_type' );

			if ( empty( $post_type ) || ( is_array( $post_type ) && ! in_array( 'product', $post_type, true ) ) || 'product' !== $post_type ) {
				return false;
			}

			$result = false;
			$atts   = $this->get_shortcode_atts();
			if ( ! empty( $atts ) && is_array( $atts ) ) {
				$limit    = $atts['limit'] ?? -1; // -1 mean display all products for shortcode
				$per_page = $q->get( 'posts_per_page' );
				if ( ! empty( $limit ) && ! empty( $per_page ) ) {
					if ( absint( $limit ) === absint( $per_page ) ) {
						$result = true;
					}
				}
			}

			/**
			 * Develop can do another check.
			 * To be sure this is the wp_query requires to hook
			 *
			 * @param bool $result
			 * @param int $filter_id
			 * @param object wp_query
			 * @param array shortcode attributes
			 */
			$result = apply_filters( 'pwf_check_is_that_custom_query_requires', $result, $this->filter_id, $q, $atts );

			return $result;
		}

		/**
		 * check shortcode types and return its attributes
		 * @since 1.5.4
		 *
		 * @return array shortcode $atts
		 */
		protected function get_shortcode_atts() {
			$atts    = array();
			$page_id = $this->global_args['current_page_id'];

			switch ( $this->filter_settings['shortcode_type'] ) {
				case 'default_woocommerce':
					$atts = Pwf_Parse_Query_Vars::extract_shortcode_attrs( $this->filter_settings['shortcode_string'] );
					break;
				case 'elementor_pro':
					$atts = Pwf_Elementor_Pro::get_widget_date( $page_id );
					if ( ! empty( $atts ) ) {
						$atts['limit'] = $atts['rows'] * $atts['columns'];
					}
					break;
				case 'powerpack_for_elementor':
					$atts = Pwf_Powerpack_Elements::get_widget_date( $page_id );
					if ( ! empty( $atts ) ) {
						$atts['limit'] = $atts['products_per_page'];
					}
					break;
			}

			return $atts;
		}

		/**
		 * Custom query used to filter products by price.
		 *
		 * @since 3.6.0
		 *
		 * @param array    $args Query args.
		 * @param WC_Query $wp_query WC_Query object.
		 *
		 * @return array
		 */
		public function price_filter_post_clauses( $args, $wp_query ) {
			$price     = self::$query_vars->get_current_min_max_price();
			$min_price = floatval( wp_unslash( $price[0] ) );
			$max_price = floatval( wp_unslash( $price[1] ) );

			$args['join']   = $this->append_product_sorting_table_join( $args['join'] );
			$args['where'] .= Pwf_Db_Utilities::get_price_where_sql( $min_price, $max_price );

			return $args;
		}

		/**
		 * Join wc_product_meta_lookup to posts if not already joined.
		 *
		 * @param string $sql SQL join.
		 * @return string
		 */
		protected function append_product_sorting_table_join( $sql ) {
			if ( ! strstr( $sql, 'wc_product_meta_lookup' ) ) {
				$sql .= Pwf_Db_Utilities::get_price_join_sql();
			}
			return $sql;
		}

		/**
		 * @since 1.0.0, 1.1.4
		 */
		protected function get_active_filter_items() {
			$data         = array();
			$filter_items = $this->get_filter_items_url_key();
			if ( empty( $filter_items ) ) {
				return $data;
			}

			if ( isset( $GLOBALS['pwf_main_query']['selected_options'] ) ) {
				$data = self::get_selected_options_from_pretty_links( $filter_items );
			} else {
				// check what item is active
				foreach ( $filter_items as $item ) {
					if ( 'priceslider' === $item['item_type'] && 'two' === $item['price_url_format'] ) {
						if ( isset( $_GET[ $item['url_key_min_price'] ] ) && isset( $_GET[ $item['url_key_max_price'] ] ) ) {
							$data[ $item['url_key'] ] = array( $_GET[ $item['url_key_min_price'] ], $_GET[ $item['url_key_max_price'] ] );
						}
					} elseif ( 'rangeslider' === $item['item_type'] && 'two' === $item['range_slider_url_format'] ) {
						if ( isset( $_GET[ $item['url_key_range_slider_min'] ] ) && isset( $_GET[ $item['url_key_range_slider_max'] ] ) ) {
							$data[ $item['url_key'] ] = array( $_GET[ $item['url_key_range_slider_min'] ], $_GET[ $item['url_key_range_slider_max'] ] );
						}
					} elseif ( 'date' === $item['item_type'] ) {
						if ( isset( $_GET[ $item['url_key_date_after'] ] ) && isset( $_GET[ $item['url_key_date_before'] ] ) ) {
							$data[ $item['url_key'] ] = array( $_GET[ $item['url_key_date_after'] ], $_GET[ $item['url_key_date_before'] ] );
						}
					} elseif ( isset( $_GET[ $item['url_key'] ] ) ) {
						$data[ $item['url_key'] ] = $_GET[ $item['url_key'] ];
					}
				}
			}

			return $data;
		}

		protected function get_filter_items_url_key() {
			return Pwf_Parse_Query_Vars::get_filter_items_without_columns( $this->filter_items );
		}

		/**
		 * Get selected options if we use pretty links
		 * @see class pwf_parse_request
		 *
		 * @param array $filter_items
		 *
		 * @since 1.5.7
		 *
		 * @return array validated selected options
		 */
		protected static function get_selected_options_from_pretty_links( $filter_items ) {
			$result           = array();
			$selected_options = $GLOBALS['pwf_main_query']['selected_options'];
			foreach ( $filter_items as $item ) {
				$url_key = $item['url_key'];
				if ( isset( $selected_options[ $url_key ] ) ) {
					$result[ $url_key ] = $selected_options[ $url_key ];
				}
			}

			return $result;
		}

		/**
		 * @since 1.3.4
		 */
		public function set_stock_status_product_variations( $where, $query_obj ) {
			$product_ids  = self::$query_vars->get_out_of_stock_ptoduct_variations_ids();
			$stock_status = self::$query_vars->get_current_stock_status();
			$where       .= Pwf_Db_Utilities::get_out_of_stock_prdoucts_variations_where_sql( $product_ids, $stock_status );

			return $where;
		}

		/**
		 * @since 1.4.7
		 */
		public function append_on_sale_products( $where, $query_obj ) {
			$where .= Pwf_Db_Utilities::get_on_sale_where_sql();
			return $where;
		}

		public function remove_product_query_filters( $posts ) {
			$this->add_anlaytic_data();

			if ( self::$hook_price ) {
				remove_filter( 'posts_clauses', array( $this, 'price_filter_post_clauses' ) );
			}

			if ( null !== self::$query_vars ) {
				if ( ! empty( self::$query_vars->has_on_sale() ) ) {
					remove_filter( 'posts_where', array( $this, 'append_on_sale_products' ), 10, 2 );
				}
				if ( self::$query_vars->is_stock_status_active() ) {
					remove_filter( 'posts_where', array( $this, 'set_stock_status_product_variations' ), 10, 2 );
				}
				if ( self::$hook_orderby ) {
					WC()->query->remove_ordering_args();
				}
			}

			return $posts;
		}

		public function search_by_title_only( $search ) {
			return Pwf_Db_Utilities::get_search_where_sql( self::$query_vars->get_search_query(), $this->filter_id, self::$query_vars, 'only_title' );
		}

		public function add_anlaytic_data() {
			global $wp_query;

			if ( self::$is_analytic_add ) {
				return;
			}

			$selected_items = self::$query_vars->selected_items();
			if ( empty( $selected_items ) ) {
				return;
			}

			$anlaytic = get_option( 'pwf_shop_analytics', 'disable' );
			if ( 'disable' === $anlaytic ) {
				return;
			}

			$orderby = self::$query_vars->get_products_orderby();
			if ( ! empty( $orderby ) ) {
				$orderby = is_array( $orderby ) ? implode( ',', $orderby ) : $orderby;
			} elseif ( isset( $_GET['orderby'] ) ) {
				$orderby = esc_attr( $_GET['orderby'] );
			}

			if ( ! empty( $orderby ) ) {
				$selected_items['orderby'] = array(
					'values' => array( $orderby ),
					'type'   => 'orderby',
				);
			}

			$filter_data = array(
				'filter_post_id' => $this->filter_id,
				'products_count' => $wp_query->found_posts,
				'from'           => 1,
				'query_string'   => self::$query_vars->get_query_string(),
			);

			$analytic_data = array(
				'filter_data'     => $filter_data,
				'selected_values' => $selected_items,
			);

			self::$is_analytic_add = true;

			$analytic = new Pwf_Analytic_Query( $analytic_data );
		}
	}

	$pwf_woo_main_query = Pwf_Woo_Main_Query::get_instance();
}
