<?php

defined( 'ABSPATH' ) || exit;

if ( ! class_exists( 'Pwf_Wpml' ) ) {

	/**
	 * Group All function relate to the plugin WPML at one place
	 *
	 * @ since 1.5.5
	 */
	class Pwf_Wpml {

		public function __construct() {

		}

		public static function get_language() {
			return apply_filters( 'wpml_current_language', null );
		}

		/**
		 * Fix Multi Language website
		 * Get transalted Filer if exist
		 * @param int $filter_id
		 * @since 1.5.5
		 *
		 * return array meta data
		 */
		public static function get_filter_post_meta( $filter_id ) {
			$filter_id = absint( $filter_id );
			$filter_id = apply_filters( 'wpml_object_id', $filter_id, 'pwf_woofilter', true, self::get_language() );
			$meta      = get_post_meta( $filter_id, '_pwf_woo_post_filter', true );

			if ( ! $meta ) {
				$meta = array();
			}

			return $meta;
		}

		/**
		 * For Multi language used to convert orginal term to translated term
		 * useful for colorlist, checkbox list
		 *
		 * @param array $item filter item settings
		 * @param array $terms the options display on the frontend
		 * @param string $meta_key colos, boxlistlabel
		 *
		 * @since ver 1.5.5
		 *
		 * @return array
		 */
		public static function convert_term_ids_to_translated_term_ids( $item, $term_type, $meta_key ) {
			foreach ( $item[ $meta_key ] as $key => $term ) {
				$item[ $meta_key ][ $key ]['term_id'] = apply_filters( 'wpml_object_id', absint( $term['term_id'] ), $term_type, true, self::get_language() );
			}

			return $item;
		}

		/**
		 * For Multi language used to convert orginal include or exclude term IDs to translated term IDs
		 * Convert Include/Exclude term IDS to translated term IDs
		 * for include & exclude terms
		 *
		 * @param array $term_ids Term Ids need to get translated term IDS
		 * @param string $term_type product_cat or taxonomy name
		 *
		 * @since ver 1.5.5
		 *
		 * @return array
		 */
		public static function convert_include_term_ids_to_translated_term_ids( $term_ids, $term_type ) {
			$result = array();
			foreach ( $term_ids as $id ) {
				$id = apply_filters( 'wpml_object_id', $id, $term_type, true, self::get_language() );
				array_push( $result, $id );
			}

			return $result;
		}

		/**
		 * Get translated term by id
		 * @param int $term_id
		 * @param string term_type product_cat, product_tag, page ..
		 *
		 * @return int translated term ID
		 */
		public static function get_translated_term_id( $term_id, $type ) {
			$term_id = apply_filters( 'wpml_object_id', $term_id, $type, true, self::get_language() );

			return $term_id;
		}

	}
}
